package rip.iwakura.civil;

import java.sql.SQLException;
import java.util.List;

import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

import com.mojang.brigadier.arguments.StringArgumentType;

import club.minnced.discord.webhook.WebhookClient;
import club.minnced.discord.webhook.WebhookClientBuilder;
import club.minnced.discord.webhook.send.WebhookMessageBuilder;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Webhook;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import net.dv8tion.jda.api.interactions.commands.DefaultMemberPermissions;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import net.dv8tion.jda.api.requests.GatewayIntent;
import net.dv8tion.jda.api.utils.messages.MessageCreateBuilder;
import rip.iwakura.civil.discord.LinkCommand;
import rip.iwakura.civil.types.CivilPlayer;
import rip.iwakura.civil.types.CivilTeam;

/**
 * Discord
 */
public class Discord extends ListenerAdapter {
    private Core core;
    public JDA jda;

    public Discord(Core core) {
        this.core = core;
    }

    public void sendMessage(CivilPlayer p, String message) {
        String webhook = p.getTeam().getWebhook();
        if (webhook != null) {
            WebhookClient client = WebhookClient.withUrl(webhook);
            client.send(new WebhookMessageBuilder().setUsername(p.getName())
                    .setAvatarUrl("https://www.mc-heads.net/avatar/" + p.getName()).setContent(message).build());
        }
    }

    public void connect(String token) {
        jda = JDABuilder.createDefault(token)
                .enableIntents(GatewayIntent.MESSAGE_CONTENT, GatewayIntent.GUILD_MESSAGES, GatewayIntent.GUILD_MEMBERS)
                .addEventListeners(this)
                .addEventListeners(new LinkCommand(core))
                .build();

        try {
            jda.awaitReady();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        registerCommands(core.getConfig().getLong("discord.guild"));
    }

    public void registerCommands(Long guildId) {
        Guild guild = jda.getGuildById(guildId);

        if (guild == null) {
            core.getLogger().info("Guild ID not found.");
            return;
        }

        guild.updateCommands().addCommands(Commands.slash("link", "Link in-game team chat to a discord channel.")
                .setDefaultPermissions(DefaultMemberPermissions.enabledFor(Permission.MANAGE_CHANNEL))
                .addOption(OptionType.STRING, "team", "Name of the team you want to link to the current channel.", true,
                        true))
                .queue();
    }

    @Override
    public void onMessageReceived(MessageReceivedEvent event) {
        if (event.getAuthor().isBot()) return;

        try {
            List<CivilTeam> linked_teams = core.database.getAllTeams(event.getChannel().getIdLong());

            for (CivilTeam team : linked_teams) {
                List<CivilPlayer> members = core.database.getAllPlayers(team);

                for (CivilPlayer member : members) {
                    Player player = Bukkit.getPlayerExact(member.getName());

                    if (player == null || !player.isOnline()) continue;

                    player.sendMessage(core.renderTeamChat(event.getAuthor(),event.getMessage().getContentDisplay()));
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }
}
