package rip.iwakura.civil.commands;

import java.sql.SQLException;
import java.util.List;
import java.util.concurrent.CompletableFuture;

import org.bukkit.entity.Player;

import com.mojang.brigadier.Message;
import com.mojang.brigadier.arguments.ArgumentType;
import com.mojang.brigadier.arguments.StringArgumentType;
import com.mojang.brigadier.context.CommandContext;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import com.mojang.brigadier.exceptions.SimpleCommandExceptionType;
import com.mojang.brigadier.suggestion.Suggestions;
import com.mojang.brigadier.suggestion.SuggestionsBuilder;

import io.papermc.paper.command.brigadier.MessageComponentSerializer;
import io.papermc.paper.command.brigadier.argument.CustomArgumentType;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import rip.iwakura.civil.types.CivilPlayer;
import rip.iwakura.civil.Database;

/**
 * PlayerArgument
 */
public class PlayerArgument implements CustomArgumentType.Converted<CivilPlayer, String> {
    private Database database;

    public PlayerArgument(Database database) {
        this.database = database;
    }

    @Override
    public CivilPlayer convert(String playerName) throws CommandSyntaxException {
        try {
            CivilPlayer civilPlayer = database.getPlayer(playerName);

            if (civilPlayer == null) {
                final Message message = MessageComponentSerializer.message()
                        .serialize(Component.text(playerName + " doesn't exist!", NamedTextColor.RED));
                throw new CommandSyntaxException(new SimpleCommandExceptionType(message), message);
            }

            return civilPlayer;

        } catch (SQLException e) {
            final Message message = MessageComponentSerializer.message()
                    .serialize(Component.text("Uh oh, an error occured!", NamedTextColor.RED));
            throw new CommandSyntaxException(new SimpleCommandExceptionType(message), message);
        }
    }

    @Override
    public <S> CompletableFuture<Suggestions> listSuggestions(CommandContext<S> ctx, SuggestionsBuilder builder) {
        try {
            List<CivilPlayer> players = database.getAllPlayers();

            for (CivilPlayer player : players) {
                String name = player.getName();

                if (name.toLowerCase().startsWith(builder.getRemainingLowerCase())) {
                    builder.suggest(name);
                }
            }
        } catch (SQLException e) {
        }

        return builder.buildFuture();
    }

    @Override
    public ArgumentType<String> getNativeType() {
        return StringArgumentType.word();
    }
}
