package rip.iwakura.civil.commands;

import java.sql.SQLException;
import java.text.Format;

import org.bukkit.Location;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import com.mojang.brigadier.Command;
import com.mojang.brigadier.arguments.LongArgumentType;
import com.mojang.brigadier.arguments.StringArgumentType;
import com.mojang.brigadier.builder.LiteralArgumentBuilder;
import com.mojang.brigadier.tree.LiteralCommandNode;

import io.papermc.paper.command.brigadier.CommandSourceStack;
import io.papermc.paper.command.brigadier.Commands;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextDecoration;
import net.kyori.adventure.text.minimessage.MiniMessage;
import rip.iwakura.civil.Core;
import rip.iwakura.civil.Database;
import rip.iwakura.civil.exceptions.InvalidChannelException;
import rip.iwakura.civil.types.*;

public class Team {
    private Database database;
    private Core core;

    public Team(Core core) {
        this.core = core;
        this.database = core.database;
    }

    private LiteralArgumentBuilder<CommandSourceStack> createTeam() {
        return Commands.literal("create")
                .requires(source -> source.getSender().hasPermission("civil.team.create"))
                .then(
                        Commands.argument("name", StringArgumentType.string())
                                .then(
                                        Commands.argument("prefix", StringArgumentType.string())
                                                .executes(ctx -> {
                                                    String name = StringArgumentType.getString(ctx, "name");
                                                    String prefix = StringArgumentType.getString(ctx, "prefix");
                                                    CommandSender sender = ctx.getSource().getSender();

                                                    try {
                                                        database.createTeam(name, prefix);
                                                        sender.sendMessage(
                                                                Component
                                                                        .text("Successfully created team ",
                                                                                NamedTextColor.GREEN)
                                                                        .append(Component
                                                                                .text(name, NamedTextColor.GOLD))
                                                                        .append(Component.text(" with prefix: \""))
                                                                        .append(MiniMessage.miniMessage()
                                                                                .deserialize(prefix))
                                                                        .append(Component.text("\"")));

                                                        return Command.SINGLE_SUCCESS;
                                                    } catch (SQLException e) {
                                                        ctx.getSource().getSender().sendMessage(
                                                                Component.text(e.getMessage(), NamedTextColor.RED));
                                                        e.printStackTrace();
                                                        return 0;
                                                    }
                                                })));
    }

    // Send team chat message
    private LiteralArgumentBuilder<CommandSourceStack> homeCommand() {
        return Commands.literal("home")
                .requires(sender -> sender.getSender() instanceof Player)
                .executes(ctx -> {
                    Player p = (Player) ctx.getSource().getSender();

                    try {
                        CivilPlayer cPlayer = core.database.getPlayer(p);
                        p.teleport(cPlayer.getTeam().getHome(core));
                        p.sendRichMessage("<green>You have been teleported to your team's home.");
                        return Command.SINGLE_SUCCESS;
                    } catch (SQLException e) {
                        e.printStackTrace();
                    }

                    return 0;
                }).then(Commands.literal("set")
                        .requires(sender -> sender.getSender() instanceof Player)
                        .executes(ctx -> {
                            Player p = (Player) ctx.getSource().getSender();

                            try {
                                CivilPlayer cPlayer = database.getPlayer(p);

                                Location location = p.getLocation();

                                database.setHome(cPlayer.getTeam(), location);

                                p.sendRichMessage("<green>Your team's home location has been set to: "
                                        + String.format("<red>%d %d %d", Math.round(location.x()),
                                                Math.round(location.y()), Math.round(location.z())));
                                return Command.SINGLE_SUCCESS;

                            } catch (SQLException e) {
                                e.printStackTrace();
                            }

                            return 1;
                        }));

    }

    // Send team chat message
    private LiteralArgumentBuilder<CommandSourceStack> teamChat() {
        return Commands.literal("chat")
                .requires(sender -> sender.getSender() instanceof Player)
                .executes(ctx -> {
                    Player p = (Player) ctx.getSource().getSender();

                    if (core.teamChatToggled.contains(p)) {
                        core.teamChatToggled.remove(p);
                        p.sendRichMessage("<red>Team chat disabled.");
                    } else {
                        core.teamChatToggled.add(p);
                        p.sendRichMessage("<green>Team chat enabled.");
                    }

                    return Command.SINGLE_SUCCESS;
                }).then(Commands.argument("message", StringArgumentType.greedyString())
                        .requires(sender -> sender.getSender() instanceof Player)
                        .executes(ctx -> {
                            CommandSender sender = ctx.getSource().getSender();
                            String message = StringArgumentType.getString(ctx, "message");

                            try {
                                CivilPlayer p = database.getPlayer((Player) sender);

                                core.sendTeamMessage(p, message);

                                return Command.SINGLE_SUCCESS;

                            } catch (SQLException e) {
                                sender.sendRichMessage("<red>Uh oh, something went wrong!");
                                e.printStackTrace();
                            }

                            return 1;
                        }));

    }

    // Add a player to a team.
    private LiteralArgumentBuilder<CommandSourceStack> joinTeam() {
        return Commands.literal("add")
                .requires(source -> source.getSender().hasPermission("civil.team.add"))
                .then(
                        Commands.argument("player", new PlayerArgument(database))
                                .then(
                                        Commands.argument("team", new TeamArgument(database))
                                                .executes(ctx -> {
                                                    CivilTeam civilTeam = ctx.getArgument("team", CivilTeam.class);
                                                    CivilPlayer civilPlayer = ctx.getArgument("player",
                                                            CivilPlayer.class);
                                                    CommandSender sender = ctx.getSource().getSender();

                                                    try {
                                                        database.joinTeam(civilPlayer, civilTeam);

                                                        sender.sendMessage(
                                                                Component
                                                                        .text("Successfully added ",
                                                                                NamedTextColor.GREEN)
                                                                        .append(Component
                                                                                .text(civilPlayer.getName(),
                                                                                        NamedTextColor.GOLD))
                                                                        .append(Component.text(" to team "))
                                                                        .append(Component
                                                                                .text(civilTeam.getName(),
                                                                                        NamedTextColor.AQUA)));
                                                        return Command.SINGLE_SUCCESS;
                                                    } catch (SQLException e) {
                                                        ctx.getSource().getSender().sendMessage(
                                                                Component.text(e.getMessage(), NamedTextColor.RED));
                                                        e.printStackTrace();
                                                        return 0;
                                                    }
                                                })));
    };

    public LiteralCommandNode<CommandSourceStack> getCommand() {
        return Commands.literal("team")
                .then(createTeam())
                .then(joinTeam())
                .then(teamChat())
                .then(homeCommand())
                .build();
    }

}
